#####################################################
# per prima cosa controlla che sia definito
# tutto quello che deve essere definito
#####################################################
macro isdefined(var)
	quote
		try
			local _ = $(esc(var))
			true
		catch err
			isa(err, UndefVarError) ? false : rethrow(err)
		end
	end
end
function check_things()
	if ~(@isdefined n) || ~(@isdefined m) || ~(@isdefined p)
		if ~(@isdefined n) 
			println("ERROR: apparently the number of              variables n is not defined!! n MUST be >  0")
		end
		if ~(@isdefined m) 
			println("ERROR: apparently the number of inequality constraints m is not defined!! m MUST be >= 0")
		end
		if ~(@isdefined p) 
			println("ERROR: apparently the number of   equality constraints p is not defined!! p MUST be >= 0")
		end
		return false
	elseif ~(@isdefined f) || ~(@isdefined g) || ~(@isdefined h)
		if ~(@isdefined f) 
			println("ERROR: apparently the              objective function f(x) is not defined!!")
		end
		if ~(@isdefined g) 
			println("ERROR: apparently the inequality constraints function g(x) is not defined!!")
		end
		if ~(@isdefined h) 
			println("ERROR: apparently the   equality constraints function h(x) is not defined!!")
		end
		return false
	elseif ~(typeof(n) <: Number) || ~(typeof(m) <: Number) || ~(typeof(p) <: Number)
		if ~(typeof(n) <: Number)
			println("ERROR: type of n is ",typeof(n))
			println("ERROR: n MUST be an integer and >  0")
		end
		if ~(typeof(m) <: Number)
			println("ERROR: type of m is ",typeof(m))
			println("ERROR: m MUST be an integer and >= 0")
		end
		if ~(typeof(p) <: Number)
			println("ERROR: type of p is ",typeof(p))
			println("ERROR: p MUST be an integer and >= 0")
		end
		return false
	elseif ~isinteger(n) || ~isinteger(m) || ~isinteger(p)
		if ~isinteger(n)
			println("ERROR: n MUST be integer and >  0")
		end
		if ~isinteger(m)
			println("ERROR: m MUST be integer and >= 0")
		end
		if ~isinteger(p)
			println("ERROR: p MUST be integer and >= 0")
		end
		return false
	elseif (n <= 0) || (m < 0) || (p < 0)
		if n <= 0
			println("ERROR: n MUST be an integer >  0")
		end
		if m < 0
			println("ERROR: m MUST be an integer >= 0")
		end
		if p < 0
			println("ERROR: p MUST be an integer >= 0")
		end
		return false
	else
		return true
	end
end

if ~check_things()
	return nothing
end
#####################################################
# definizione di gradienti di f.o. e vincoli
#####################################################
"""
returns the gradient of the objective function 
f(x) as a column vector with n components
"""
function nabla_f(x)
	return ForwardDiff.gradient(f,x)
end

"""
returns the gradient of the constraint function 
g(x) as a n x m matrix. Gradients of the constraints
are stored in the columns of the matrix. If m = 0 returns
an empty matrix
"""
function nabla_g(x)
	if m > 0
		return ForwardDiff.jacobian(g,x)'
	else
		return zeros(m,n)
	end
end

"""
returns the gradient of the constraint function 
h(x) as a n x p matrix. Gradients of the constraints
are stored in the columns of the matrix. If p = 0 returns
an empty matrix
"""
function nabla_h(x)
	if p > 0
		return ForwardDiff.jacobian(h,x)'
	else
		return zeros(p,n)
	end
end

#####################################################
# definizione di hessiani di f.o. e vincoli
#####################################################
"""
returns the hessian of the objective function f(x)
as a n x n matrix.
"""
function hess_f(x)
	return ForwardDiff.hessian(f,x)
end
"""
returns the hessians of the constraint functions g(x)
as an array of n x n matrices. Hence, hessian of the 
i-th constraint is stored in hess_g(x)[i].
If m == 0 returns an empty array of arrays.
"""
function hess_g(x)
	if m > 0
		hg = Array[]
		for i in 1:m
			gi(x) = g(x)[i]
			push!(hg,ForwardDiff.hessian(gi,x))
		end
		return hg
	else
		return Array[]
	end
end
"""
returns the hessians of the constraint functions h(x)
as an array of n x n matrices. Hence, hessian of the 
i-th constraint is stored in hess_h(x)[i].
If p == 0 returns an empty array of arrays.
"""
function hess_h(x)
	if p > 0
		hg = Array[]
		for i in 1:p
			hi(x) = h(x)[i]
			push!(hg,ForwardDiff.hessian(hi,x))
		end
		return hg
	else
		return Array[]
	end
end

#####################################################
# definizione di L, nabla_L e hess_L
#####################################################
"""
returns value of the Lagrangian function. L(x,la,mu) is
defined as L = f(x) + sum_{i=1}^m la[i]*g(x)[i] + 
sum_{i=1}^p mu[i]*h(x)[i]
"""
function L(x,la,mu)
	res = f(x)
	gx  = g(x)
	hx  = h(x)
	for i = 1:m
		res += gx[i]*la[i] 
	end
	for i = 1:p
		res += hx[i]*mu[i] 
	end
	return res
end
"""
returns the gradient of the Lagrangian function w.r.t. the (primal) 
variables x, i.e. a column vector of n components.
nabla_L is defined as nabla_f(x) + sum_{i=1}^m la[i]*nabla_g(x)[:,i] + 
sum_{i=1}^p mu[i]*nabla_h(x)[:,i] which is equivalent to 
nabla_f(x) + nabla_g(x)*la + nabla_h(x)*mu where '*' is to be intended
as the matrix-vector product
"""
function nabla_L(x,la,mu)
	res = nabla_f(x)
	if p > 0
		res += nabla_h(x)*mu
	end
	if m > 0
		res += nabla_g(x)*la
	end
	return res
end
"""
returns the hessian of the Lagrangian function w.r.t. the (primal)
variables x, i.e. a n x n matrix.
hess_L is defined as hess_f(x) + sum_{i=1}^m la[i]*hess_g(x)[i] + 
sum_{i=1}^p mu[i]*hess_h(x)[i]
"""
function hess_L(x,la,mu)
	res = hess_f(x)
	hg = hess_g(x)
	hh = hess_h(x)
	for i = 1:m
		res += hg[i]*la[i]
	end
	for i = 1:p
		res += hh[i]*mu[i]
	end
	return res
end

#####################################################
# definizione di sistema KKT e suo jacobiano
#####################################################
"""
returns the residual vector of the KKT system. The first n 
components is nabla_L(x,la,mu). Then, there are the components
of the active inequality constraints and, finally, the equality 
constraints. Here, 
iact denotes the indices of the (stimated) active inequality 
constraints
"""
function F(x,la,mu,iact)
	res = nabla_L(x,la,mu)
	mact = length(iact)
	if mact > 0
		res = [res; g(x)[iact]]
	end
	if p > 0
		res = [res; h(x)]
	end
	return res
end
"""
returns the Jacobian of F(x,la,mu,iact) as the following (big) matrix

``
  JF(x,la,mu,iact) = \\left(\\begin{array}[ccc]
                     \\nabla^2 L     \\nabla g[:,iact]      \\nabla h    \\\\
                 \\nabla g[:,iact]'  zeros(mact,mact)  zeros(mact,p) \\\\
                 \\nabla h'           zeros(p,mact)     zeros(p,p)    
                     \\end{array}\\right)
``

"""
function JF(x,la,mu,iact)
	H = hess_L(x,la,mu)
	v = Array{Float64}(n,0)
	mact = length(iact)
	if mact > 0
		v = [v nabla_g(x)[:,iact]]
	end
	if p > 0
		v = [v nabla_h(x)]
	end
	return [H v;v' zeros(mact+p,mact+p)]
end

nothing
